@@ -24,13 +24,24 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include "dialog.h"
+#include "fileutils.h"
+#include "globals.h"
 #include "uibuilder.h"
 #include "vikradiogroup.h"
 #include "vikfileentry.h"
 #include "vikfilelist.h"
+#include "ui_util.h"
 
 VikLayerParamData vik_lpd_true_default ( void ) { return VIK_LPD_BOOLEAN ( TRUE ); }
 VikLayerParamData vik_lpd_false_default ( void ) { return VIK_LPD_BOOLEAN ( FALSE ); }
+static gboolean file_save_cb ( VikFileEntry *vfe, gpointer user_data );
+
+/** i18n note
+ * Since UI builder often uses static structures, the text is marked with N_()
+ *  however to actually get it to apply the widget (e.g. to a label) then
+ *  an additional call to _() needs to occur on that string
+ **/
 
 GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data )
 {
@@ -62,7 +73,7 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
         gchar **pstr = param->widget_data;
         rv = vik_combo_box_text_new ();
         while ( *pstr )
-          vik_combo_box_text_append ( rv, *(pstr++) );
+          vik_combo_box_text_append ( rv, _(*(pstr++)) );
         if ( param->extra_widget_data ) /* map of alternate uint values for options */
         {
           /* Set the effective default value */
@@ -88,9 +99,9 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
         rv = gtk_combo_box_entry_new_text ();
 #endif
         if ( vlpd.s )
-          vik_combo_box_text_append ( rv, vlpd.s );
+          vik_combo_box_text_append ( rv, _(vlpd.s) );
         while ( *pstr )
-          vik_combo_box_text_append ( rv, *(pstr++) );
+          vik_combo_box_text_append ( rv, _(*(pstr++)) );
         if ( vlpd.s )
           gtk_combo_box_set_active ( GTK_COMBO_BOX ( rv ), 0 );
       }
@@ -100,7 +111,7 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
         gchar **pstr = param->widget_data;
         rv = GTK_WIDGET ( vik_combo_box_text_new () );
         while ( *pstr )
-          vik_combo_box_text_append ( rv, *(pstr++) );
+          vik_combo_box_text_append ( rv, _(*(pstr++)) );
         if ( vlpd.s )
         {
           /* Set the effective default value */
@@ -169,7 +180,7 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
     case VIK_LAYER_WIDGET_ENTRY:
       if ( param->type == VIK_LAYER_PARAM_STRING )
       {
-        rv = gtk_entry_new ();
+        rv = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
         if ( vlpd.s )
           gtk_entry_set_text ( GTK_ENTRY(rv), vlpd.s );
       }
@@ -177,7 +188,7 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
     case VIK_LAYER_WIDGET_PASSWORD:
       if ( param->type == VIK_LAYER_PARAM_STRING )
       {
-        rv = gtk_entry_new ();
+        rv = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
         gtk_entry_set_visibility ( GTK_ENTRY(rv), FALSE );
         if ( vlpd.s )
           gtk_entry_set_text ( GTK_ENTRY(rv), vlpd.s );
@@ -193,6 +204,14 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
           vik_file_entry_set_filename ( VIK_FILE_ENTRY(rv), vlpd.s );
       }
       break;
+    case VIK_LAYER_WIDGET_FILESAVE:
+      if ( param->type == VIK_LAYER_PARAM_STRING )
+      {
+        rv = vik_file_entry_new (GTK_FILE_CHOOSER_ACTION_SAVE, GPOINTER_TO_INT(param->widget_data), file_save_cb, NULL );
+        if ( vlpd.s )
+          vik_file_entry_set_filename ( VIK_FILE_ENTRY(rv), vlpd.s );
+      }
+      break;
     case VIK_LAYER_WIDGET_FOLDERENTRY:
       if ( param->type == VIK_LAYER_PARAM_STRING )
       {
@@ -219,10 +238,11 @@ GtkWidget *a_uibuilder_new_widget ( VikLayerParam *param, VikLayerParamData data
         gtk_scale_set_digits ( GTK_SCALE(rv), scale->digits );
         gtk_range_set_value ( GTK_RANGE(rv), init_val );
       }
+      break;
 
     case VIK_LAYER_WIDGET_BUTTON:
       if ( param->type == VIK_LAYER_PARAM_PTR && param->widget_data ) {
-        rv = gtk_button_new_with_label ( param->widget_data );
+        rv = gtk_button_new_with_label ( _(param->widget_data) );
         g_signal_connect ( G_OBJECT(rv), "clicked", G_CALLBACK (vlpd.ptr), param->extra_widget_data );
       }
       break;
@@ -295,6 +315,7 @@ VikLayerParamData a_uibuilder_widget_get_value ( GtkWidget *widget, VikLayerPara
       rv.s = gtk_entry_get_text ( GTK_ENTRY(widget) );
       break;
     case VIK_LAYER_WIDGET_FILEENTRY:
+    case VIK_LAYER_WIDGET_FILESAVE:
     case VIK_LAYER_WIDGET_FOLDERENTRY:
       rv.s = vik_file_entry_get_filename ( VIK_FILE_ENTRY(widget) );
       break;
@@ -319,20 +340,46 @@ VikLayerParamData a_uibuilder_widget_get_value ( GtkWidget *widget, VikLayerPara
   return rv;
 }
 
-//static void draw_to_image_file_total_area_cb (GtkSpinButton *spinbutton, gpointer *pass_along)
+static GtkWidget *dialog = NULL;
+
+/**
+ * Hacky method to enable closing the dialog within preference code
+ */
+void a_uibuilder_factory_close ( gint response_id )
+{
+  if ( dialog ) {
+    gtk_dialog_response ( GTK_DIALOG(dialog), response_id );
+  }
+}
+
+/**
+ * @have_apply_button: Whether the dialog should have an apply button
+ * @redraw:            Function to be invoked to redraw when apply button is pressed
+ *                     Typically expected to be #vik_layer_emit_update()
+ * @redraw_param:      Parameter to be passed to the redraw(), so typically a VikLayer*
+ *
+ * Returns:
+ *  0 = Dialog cancelled
+ *  1 = No parameters to be displayed
+ *  2 = Parameter changed that needs redraw
+ *  3 = Parameter changed but redraw not necessary
+ */
 gint a_uibuilder_properties_factory ( const gchar *dialog_name,
                                       GtkWindow *parent,
                                       VikLayerParam *params,
                                       guint16 params_count,
                                       gchar **groups,
                                       guint8 groups_count,
-                                      gboolean (*setparam) (gpointer,guint16,VikLayerParamData,gpointer,gboolean),
+                                      gboolean (*setparam) (gpointer,gpointer),
+                                      gboolean (*setparam4) (gpointer,guint16,VikLayerParamData,gpointer),
                                       gpointer pass_along1,
                                       gpointer pass_along2,
                                       VikLayerParamData (*getparam) (gpointer,guint16,gboolean),
                                       gpointer pass_along_getparam,
-                                      void (*changeparam) (GtkWidget*, ui_change_values) )
-                                      /* pass_along1 and pass_along2 are for set_param first and last params */
+                                      void (*changeparam) (GtkWidget*, ui_change_values),
+                                      gboolean have_apply_button,
+                                      void (*redraw) (gpointer),
+                                      gpointer redraw_param )
 {
   guint16 i, j, widget_count = 0;
   gboolean must_redraw = FALSE;
@@ -349,18 +396,18 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
   else
   {
     /* create widgets and titles; place in table */
-    GtkWidget *dialog = gtk_dialog_new_with_buttons ( dialog_name,
-						      parent,
-						      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-						      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-						      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL );
+    dialog = gtk_dialog_new_with_buttons ( dialog_name, parent,
+                                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                           GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL );
+    if ( have_apply_button )
+      gtk_dialog_add_button ( GTK_DIALOG(dialog), GTK_STOCK_APPLY, GTK_RESPONSE_APPLY );
+    gtk_dialog_add_button ( GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_ACCEPT );
+
     gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
     GtkWidget *response_w = NULL;
 #if GTK_CHECK_VERSION (2, 20, 0)
     response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 #endif
-    gint resp;
-
     GtkWidget *table = NULL;
     GtkWidget **tables = NULL; /* for more than one group */
 
@@ -377,7 +424,7 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
       // Switch to vertical notebook mode when many groups
       if ( groups_count > 4 )
         gtk_notebook_set_tab_pos ( GTK_NOTEBOOK(notebook), GTK_POS_LEFT );
-      gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), notebook, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), notebook, TRUE, TRUE, 0);
       tables = g_malloc ( sizeof(GtkWidget *) * groups_count );
       for ( current_group = 0; current_group < groups_count; current_group++ )
       {
@@ -396,7 +443,7 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
     else
     {
       table = gtk_table_new( widget_count, 1, FALSE );
-      gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), table, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), table, TRUE, TRUE, 0);
     }
 
     for ( i = 0, j = 0; i < params_count; i++ )
@@ -411,7 +458,8 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
         if ( widgets[j] ) {
           labels[j] = gtk_label_new(_(params[i].title));
           gtk_table_attach ( GTK_TABLE(table), labels[j], 0, 1, j, j+1, 0, 0, 0, 0 );
-          gtk_table_attach ( GTK_TABLE(table), widgets[j], 1, 2, j, j+1, GTK_EXPAND | GTK_FILL, 0, 2, 2 );
+          gtk_table_attach ( GTK_TABLE(table), widgets[j], 1, 2, j, j+1, GTK_EXPAND | GTK_FILL,
+                             params[i].type == VIK_LAYER_PARAM_STRING_LIST ? GTK_EXPAND | GTK_FILL : 0, 2, 2 );
 
           if ( changeparam )
           {
@@ -456,31 +504,37 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
 
     gtk_widget_show_all ( dialog );
 
-    resp = gtk_dialog_run (GTK_DIALOG (dialog));
-    if ( resp == GTK_RESPONSE_ACCEPT )
-    {
-      for ( i = 0, j = 0; i < params_count; i++ )
-      {
-        if ( params[i].group != VIK_LAYER_NOT_IN_PROPERTIES )
-        {
-          if ( setparam ( pass_along1,
-			  i,
-			  a_uibuilder_widget_get_value ( widgets[j], &(params[i]) ),
-			  pass_along2,
-			  FALSE ) )
-            must_redraw = TRUE;
-          j++;
+    gint resp = GTK_RESPONSE_APPLY;
+    gint answer = 0;
+    while ( resp == GTK_RESPONSE_APPLY ) {
+      resp = gtk_dialog_run (GTK_DIALOG (dialog));
+      if ( resp == GTK_RESPONSE_ACCEPT || resp == GTK_RESPONSE_APPLY ) {
+	VikLayerSetParam vlsp;
+	vlsp.is_file_operation = FALSE;
+	vlsp.dirpath = NULL;
+	for ( i = 0, j = 0; i < params_count; i++ ) {
+	  if ( params[i].group != VIK_LAYER_NOT_IN_PROPERTIES ) {
+            vlsp.id = i;
+            vlsp.vp = pass_along2;
+            vlsp.data = a_uibuilder_widget_get_value ( widgets[j], &(params[i]) );
+            // Main callback into each layer's setparam
+            if ( setparam && setparam ( pass_along1, &vlsp ) )
+              must_redraw = TRUE;
+            // Or a basic callback for each parameter
+            else if ( setparam4 && setparam4 ( pass_along1, i, vlsp.data, pass_along2 ) )
+              must_redraw = TRUE;
+            j++;
+          }
         }
+
+        if ( resp == GTK_RESPONSE_APPLY ) {
+          if ( redraw && redraw_param )
+            redraw ( redraw_param );
+          answer = 0;
+        }
+        else
+          answer = must_redraw ? 2 : 3; // user clicked OK
       }
-
-      g_free ( widgets );
-      g_free ( labels );
-      g_free ( change_values );
-      if ( tables )
-        g_free ( tables );
-      gtk_widget_destroy ( dialog ); /* hide before redrawing. */
-
-      return must_redraw ? 2 : 3; /* user clicked OK */
     }
 
     g_free ( widgets );
@@ -489,12 +543,12 @@ gint a_uibuilder_properties_factory ( const gchar *dialog_name,
     if ( tables )
       g_free ( tables );
     gtk_widget_destroy ( dialog );
+    dialog = NULL;
 
-    return 0;
+    return answer;
   }
 }
 
-
 static void uibuilder_run_setparam ( VikLayerParamData *paramdatas, guint16 i, VikLayerParamData data, VikLayerParam *params )
 {
   /* could have to copy it if it's a string! */
@@ -524,12 +578,16 @@ VikLayerParamData *a_uibuilder_run_dialog (  const gchar *dialog_name, GtkWindow
 					  params_count, 
 					  groups, 
 					  groups_count,
+					  NULL,
 					  (gpointer) uibuilder_run_setparam, 
 					  paramdatas, 
 					  params,
 					  (gpointer) uibuilder_run_getparam, 
 					  params_defaults,
-					  NULL ) > 0 ) {
+					  NULL,
+					  FALSE,
+					  NULL,
+					  NULL) > 0 ) {
 
       return paramdatas;
     }
@@ -563,3 +621,10 @@ void a_uibuilder_free_paramdatas ( VikLayerParamData *paramdatas, VikLayerParam
   }
   g_free ( paramdatas );
 }
+
+static gboolean file_save_cb ( VikFileEntry *vfe, gpointer user_data ) {
+  const gchar *entry = vik_file_entry_get_filename ( vfe );
+  return ( g_file_test ( entry, G_FILE_TEST_EXISTS ) == FALSE ||
+           a_dialog_yes_or_no ( VIK_GTK_WINDOW_FROM_WIDGET(vfe), _("The file \"%s\" exists, do you wish to overwrite it?"), a_file_basename ( entry ) ) );
+}
+
